@@ -1,4 +1,4 @@
-/* $Id: gweather-pref.c 10780 2008-04-07 09:10:09Z vuntz $ */
+/* $Id: gweather-pref.c 11193 2009-01-14 07:56:46Z diegoe $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -24,7 +24,6 @@
 #include <ctype.h>
 #include <locale.h>
 
-#include <gnome.h>
 #include <panel-applet.h>
 #include <gconf/gconf-client.h>
 
@@ -735,9 +734,9 @@ static void help_cb (GtkDialog *dialog)
 {
     GError *error = NULL;
 
-    gnome_help_display_on_screen (
-		"gweather", "gweather-settings",
-		gtk_window_get_screen (GTK_WINDOW (dialog)),
+    gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)),
+		"ghelp:gweather?gweather-settings",
+		gtk_get_current_event_time (),
 		&error);
 
     if (error) { 
