@@ -32,19 +32,10 @@
 #include <glib-object.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <gtk/gtkaboutdialog.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkrange.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 #include <gconf/gconf-client.h>
 
-#include <libgnomeui/gnome-help.h>
-#include <libgnome/gnome-desktop-item.h>
-
 #include "applet.h"
 #include "keys.h"
 #include "preferences.h"
@@ -79,7 +70,11 @@ static void	gnome_volume_applet_orientation	(PanelApplet *applet,
 						 PanelAppletOrient orient);
 
 static gboolean	gnome_volume_applet_refresh	(GnomeVolumeApplet *applet,
-						 gboolean           force_refresh);
+						 gboolean           force_refresh,
+						 gdouble            volume,
+						 gint               mute);
+
+static void     cb_notify_message (GstBus *bus, GstMessage *message, gpointer data);
 static gboolean	cb_check			(gpointer   data);
 
 static void	cb_volume			(GtkAdjustment *adj,
@@ -131,7 +126,8 @@ init_pixbufs (GnomeVolumeApplet *applet)
 					       applet->panel_size - 4,
 					       0,
 					       NULL);
-    if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
+    if (applet->pix[n] != NULL &&
+	gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
       GdkPixbuf *temp;
 
       temp = gdk_pixbuf_flip (applet->pix[n], TRUE);
@@ -178,6 +174,7 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   applet->state = -1;
   applet->prefs = NULL;
   applet->dock = NULL;
+  applet->adjustment = NULL;
   applet->panel_size = 24;
 
   g_set_application_name (_("Volume Applet"));
@@ -218,6 +215,13 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   /* i18n */
   ao = gtk_widget_get_accessible (GTK_WIDGET (applet));
   atk_object_set_name (ao, _("Volume Control"));
+
+  /* Watch for signals from GST. */
+  applet->bus = gst_bus_new ();
+  gst_bus_add_signal_watch (applet->bus);
+  g_signal_connect (G_OBJECT (applet->bus), "message::element",
+		    (GCallback) cb_notify_message, applet);
+
 }
 
 /* Parse the list of tracks that are stored in GConf */
@@ -246,12 +250,8 @@ select_tracks (GstElement *element,
   active_track_name_list = NULL;
 
   if (reset_state) {
-#ifdef HAVE_GST10
     gst_element_set_state (element, GST_STATE_READY);
     if (gst_element_get_state(element, NULL, NULL, -1) != GST_STATE_CHANGE_SUCCESS)
-#else
-    if (gst_element_set_state (element, GST_STATE_READY) != GST_STATE_SUCCESS)
-#endif
       return NULL;
   }
 
@@ -344,12 +344,33 @@ select_element_and_track (GnomeVolumeApplet *applet,
     return FALSE;
 
   applet->mixer = g_object_ref (active_element);
+  gst_element_set_bus (GST_ELEMENT (applet->mixer), applet->bus);
   applet->tracks = active_tracks;
   g_list_foreach (applet->tracks, (GFunc) g_object_ref, NULL);
 
   return TRUE;
 }
 
+static void
+gnome_volume_applet_setup_timeout (GnomeVolumeApplet *applet)
+{
+  gboolean need_timeout = TRUE;
+  need_timeout = ((gst_mixer_get_mixer_flags (GST_MIXER (applet->mixer)) &
+      GST_MIXER_FLAG_AUTO_NOTIFICATIONS) == 0);
+
+  if (need_timeout) {
+    if (applet->timeout == 0) {
+      applet->timeout = g_timeout_add (100, cb_check, applet);
+    }
+  }
+  else {
+    if (applet->timeout != 0) {
+      g_source_remove (applet->timeout);
+      applet->timeout = 0;
+    }
+  }
+}
+
 gboolean
 gnome_volume_applet_setup (GnomeVolumeApplet *applet,
 			   GList *elements)
@@ -386,9 +407,15 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
   if (res) {
     first_track = g_list_first (applet->tracks)->data;
 
-    /* tell the dock */
-    adj = gtk_adjustment_new (50, 0, 100, 4, 10, 0);
-    gtk_adjustment_set_value (GTK_ADJUSTMENT (adj), 
+    applet->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (50, 0, 100, 
+							     4, 10, 0));
+    /* We want a reference from the applet as well as from the dock it
+     * will be attached to. */
+    g_object_ref_sink (applet->adjustment);
+    g_signal_connect (applet->adjustment, "value-changed",
+		      G_CALLBACK (cb_volume), applet);
+
+    gtk_adjustment_set_value (applet->adjustment, 
 			      gnome_volume_applet_get_volume (applet->mixer, 
 							      first_track));
   }
@@ -396,13 +423,6 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
   gnome_volume_applet_orientation (PANEL_APPLET (applet),
 				   panel_applet_get_orient (PANEL_APPLET (applet)));
 
-  if (res) {
-    gnome_volume_applet_dock_change (applet->dock,
-				     GTK_ADJUSTMENT (adj));
-    g_signal_connect (adj, "value-changed",
-		      G_CALLBACK (cb_volume), applet);
-  }
-
   /* menu - done here because bonobo is intialized now */
   panel_applet_setup_menu_from_file (PANEL_APPLET (applet), 
 				     DATADIR,
@@ -411,12 +431,10 @@ gnome_volume_applet_setup (GnomeVolumeApplet *applet,
   component = panel_applet_get_popup_component (PANEL_APPLET (applet));
   g_signal_connect (component, "ui-event", G_CALLBACK (cb_ui_event), applet);
 
-  gnome_volume_applet_refresh (applet, TRUE);
+  gnome_volume_applet_refresh (applet, TRUE, -1, -1);
   if (res) {
-    applet->timeout = g_timeout_add (100, cb_check, applet);
-  }
+    gnome_volume_applet_setup_timeout (applet);
 
-  if (res) {
     /* gconf */
     key = panel_applet_gconf_get_full_key (PANEL_APPLET (applet),
 				GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT);
@@ -472,6 +490,16 @@ gnome_volume_applet_dispose (GObject *object)
     applet->timeout = 0;
   }
 
+  if (applet->dock) {
+    g_object_unref (applet->dock);
+    applet->dock = NULL;
+  }
+
+  if (applet->adjustment) {
+    g_object_unref (applet->adjustment);
+    applet->adjustment = NULL;
+  }
+
   for (n = 0; n < 5; n++) {
     if (applet->pix[n] != NULL) {
       g_object_unref (G_OBJECT (applet->pix[n]));
@@ -479,6 +507,12 @@ gnome_volume_applet_dispose (GObject *object)
     }
   }
 
+  if (applet->bus) {
+    gst_bus_remove_signal_watch (applet->bus);
+    gst_object_unref (applet->bus);
+    applet->bus = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -566,24 +600,20 @@ gnome_volume_applet_popup_dock (GnomeVolumeApplet *applet)
   GtkWidget *widget = GTK_WIDGET (applet);
   gint x, y;
 
-  /* show (before reposition, so size allocation is done) */
+  /* Get it in just about the right position so that it
+   * doesn't flicker to obviously when we reposition it. */
   gnome_volume_applet_get_dock_position (applet, &x, &y);
   gtk_window_move (GTK_WINDOW (applet->dock), x, y);
+
   gtk_widget_show_all (GTK_WIDGET (applet->dock));
 
-  /* reposition */
+  /* Reposition the window now that we know its actual size
+   * and can center it. */
   gnome_volume_applet_get_dock_position (applet, &x, &y);
   gtk_window_move (GTK_WINDOW (applet->dock), x, y);
 
-  /* grab input */
-  gtk_widget_grab_focus (GTK_WIDGET (applet->dock->scale));
-  gtk_grab_add (widget);
-  gdk_pointer_grab (widget->window, TRUE,
-		    GDK_BUTTON_PRESS_MASK |
-		    GDK_BUTTON_RELEASE_MASK |
-		    GDK_POINTER_MOTION_MASK,
-		    NULL, NULL, GDK_CURRENT_TIME);
-  gdk_keyboard_grab (widget->window, TRUE, GDK_CURRENT_TIME);
+  /* Set the keyboard focus in the correct place. */
+  gnome_volume_applet_dock_set_focus (applet->dock);
 
   /* set menu item as active */
   gtk_widget_set_state (GTK_WIDGET (applet), GTK_STATE_SELECTED);
@@ -600,11 +630,6 @@ gnome_volume_applet_popdown_dock (GnomeVolumeApplet *applet)
   if (!applet->pop)
     return;
 
-  /* release input */
-  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-  gtk_grab_remove (widget);
-
   /* hide */
   gtk_widget_hide_all (GTK_WIDGET (applet->dock));
 
@@ -625,16 +650,22 @@ gnome_volume_applet_pop_dock (GnomeVolumeApplet *applet)
   }
 }
 
+gboolean
+mixer_is_muted (GnomeVolumeApplet *applet)
+{
+  return applet->state & 1;
+}
+
 /*
  * Toggle mute.
  */
 
-static void
+void
 gnome_volume_applet_toggle_mute (GnomeVolumeApplet *applet)
 {
   BonoboUIComponent *component;
-  gboolean mute = applet->state & 1;
-  GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
+  gboolean mute = mixer_is_muted (applet);
+  gboolean newmute = !mute;
   GList *tracks;
 
   for (tracks = g_list_first (applet->tracks); tracks; tracks = tracks->next)
@@ -642,43 +673,37 @@ gnome_volume_applet_toggle_mute (GnomeVolumeApplet *applet)
 
   if (mute) {
     /* sync back actual volume */
-    cb_volume (adj, applet);
+    cb_volume (applet->adjustment, applet);
   }
 
   /* update component */
   component = panel_applet_get_popup_component (PANEL_APPLET (applet));
   bonobo_ui_component_set_prop (component,
 			        "/commands/Mute",
-			        "state", mute ? "1" : "0", NULL);
+			        "state", newmute ? "1" : "0", NULL);
+
+  /* Update the dock. */
+  if (applet->dock) {
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (applet->dock->mute), 
+				  newmute);
+  }
 
   /* update graphic - this should happen automagically after the next
    * idle call, but apparently doesn't for some people... */
-  gnome_volume_applet_refresh (applet, TRUE);
+  gnome_volume_applet_refresh (applet, TRUE, -1, newmute);
 }
 
 /*
  * Run g-v-c.
  */
 
-static void
+void
 gnome_volume_applet_run_mixer (GnomeVolumeApplet *applet)
 {
-  GnomeDesktopItem *ditem;
   GError *error = NULL;
 
-  if ((ditem = gnome_desktop_item_new_from_basename ("gnome-volume-control.desktop", 0, NULL))) {
-    gnome_desktop_item_set_launch_time (ditem, gtk_get_current_event_time ());
-    gnome_desktop_item_launch_on_screen (ditem, NULL,
-	                                 GNOME_DESKTOP_ITEM_LAUNCH_ONLY_ONE,
-					 gtk_widget_get_screen (GTK_WIDGET (applet)),
-					 -1, &error);
-    gnome_desktop_item_unref (ditem);
-  }
-  else {
-    gdk_spawn_command_line_on_screen (
-	      gtk_widget_get_screen (GTK_WIDGET (applet)),
-	      "gnome-volume-control", &error);
-  }
+  gdk_spawn_command_line_on_screen (gtk_widget_get_screen (GTK_WIDGET (applet)),
+				    "gnome-volume-control", &error);
 
   if (error) {
     GtkWidget *dialog;
@@ -713,20 +738,19 @@ gnome_volume_applet_scroll (GtkWidget      *widget,
     switch (event->direction) {
       case GDK_SCROLL_UP:
       case GDK_SCROLL_DOWN: {
-        GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
-        gdouble volume = adj->value;
+        gdouble volume = gtk_adjustment_get_value (applet->adjustment);
 
         if (event->direction == GDK_SCROLL_UP) {
-          volume += adj->step_increment;
-          if (volume > adj->upper)
-            volume = adj->upper;
+          volume += applet->adjustment->step_increment;
+          if (volume > applet->adjustment->upper)
+            volume = applet->adjustment->upper;
         } else {
-          volume -= adj->step_increment;
-          if (volume < adj->lower)
-            volume = adj->lower;
+          volume -= applet->adjustment->step_increment;
+          if (volume < applet->adjustment->lower)
+            volume = applet->adjustment->lower;
         }
 
-        gtk_range_set_value (applet->dock->scale, volume);
+        gtk_adjustment_set_value (applet->adjustment, volume);
         return TRUE;
       }
       default:
@@ -765,7 +789,7 @@ gnome_volume_applet_button (GtkWidget      *widget,
             if (applet->mixer) {
               gnome_volume_applet_popdown_dock (applet);
             }
-            gnome_volume_applet_run_mixer (applet);
+            gnome_volume_applet_toggle_mute (applet);
             return TRUE;
           default:
             break;
@@ -827,30 +851,30 @@ gnome_volume_applet_key (GtkWidget   *widget,
     case GDK_Right:
     case GDK_Up:
     case GDK_Down: {
-      GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
-      gdouble volume = adj->value, increment;
+      gdouble volume = gtk_adjustment_get_value (applet->adjustment);
+      gdouble increment;
 
       if (event->state != 0)
         break;
 
       if (event->keyval == GDK_Up || event->keyval == GDK_Down 
          ||event->keyval == GDK_Left)
-        increment = adj->step_increment;
+        increment = applet->adjustment->step_increment;
       else
-        increment = adj->page_increment;
+        increment = applet->adjustment->page_increment;
 
       if (event->keyval == GDK_Page_Up || event->keyval == GDK_Up
          ||event->keyval == GDK_Right) {
         volume += increment;
-        if (volume > adj->upper)
-          volume = adj->upper;
+        if (volume > applet->adjustment->upper)
+          volume = applet->adjustment->upper;
       } else {
         volume -= increment;
-        if (volume < adj->lower)
-          volume = adj->lower;
+        if (volume < applet->adjustment->lower)
+          volume = applet->adjustment->lower;
       }
 
-      gtk_range_set_value (applet->dock->scale, volume);
+      gtk_adjustment_set_value (applet->adjustment, volume);
       return TRUE;
     }
     default:
@@ -860,6 +884,15 @@ gnome_volume_applet_key (GtkWidget   *widget,
   return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
+static gboolean
+gnome_volume_applet_dock_focus_out (GtkWidget *dock, GdkEventFocus *event,
+				    GnomeVolumeApplet *applet)
+{
+  gnome_volume_applet_popdown_dock (applet);
+
+  return FALSE;
+}
+
 /*
  * Change orientation or size of panel.
  */
@@ -869,25 +902,19 @@ gnome_volume_applet_orientation	(PanelApplet *_applet,
 				 PanelAppletOrient orientation)
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (_applet);
-  GtkAdjustment *adj = NULL;
   GtkWidget *dock;
 
   if (applet->dock) {
-    adj = gtk_range_get_adjustment (applet->dock->scale);
-    g_object_ref (G_OBJECT (adj));
-    /* FIXME:
-     * - we need to unset the parent in some way, because else Gtk+
-     *    thinks that the child of the applet (image) is us (dock),
-     *    which is not the case.
-     */
-    gtk_widget_unparent (GTK_WIDGET (applet->dock));
+    gtk_widget_destroy (GTK_WIDGET (applet->dock));
   }
-  dock = gnome_volume_applet_dock_new (IS_PANEL_HORIZONTAL (orientation) ?
-      GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
-  /* parent, for signal forwarding */
-  gtk_widget_set_parent (dock, GTK_WIDGET (applet));
+  dock = gnome_volume_applet_dock_new (GTK_ORIENTATION_VERTICAL,
+				       applet);
+  gtk_widget_add_events (dock, GDK_FOCUS_CHANGE_MASK);
+  g_signal_connect (G_OBJECT (dock), "focus-out-event",
+		    G_CALLBACK (gnome_volume_applet_dock_focus_out),
+		    applet);
   applet->dock = GNOME_VOLUME_APPLET_DOCK (dock);
-  gnome_volume_applet_dock_change (applet->dock, adj);
+  gnome_volume_applet_dock_change (applet->dock, applet->adjustment);
 
   if (PANEL_APPLET_CLASS (parent_class)->change_orient)
     PANEL_APPLET_CLASS (parent_class)->change_orient (_applet, orientation);
@@ -916,7 +943,7 @@ void gnome_volume_applet_size_allocate (GtkWidget     *widget,
   }
 
   init_pixbufs (applet);
-  gnome_volume_applet_refresh (applet, TRUE);
+  gnome_volume_applet_refresh (applet, TRUE, -1, -1);
 }
 
 static void
@@ -966,15 +993,23 @@ gnome_volume_applet_adjust_volume (GstMixer *mixer,
 {
   int range = track->max_volume - track->min_volume;
   gdouble scale = ((gdouble) range) / 100;
-  int *volumes, n;
+  int *volumes, n, volint;
 
-  volume *= scale;
-  volume += track->min_volume;
+  if (volume == 1.0) {
+    volint = track->max_volume;
+  } else if (volume == 0.0) {
+    volint = track->min_volume;
+  } else {
+    volume *= scale;
+    volume += track->min_volume;
+    volint = lrint (volume);
+  }
 
   volumes = g_new (gint, track->num_channels);
   for (n = 0; n < track->num_channels; n++)
-    volumes[n] = lrint (volume);
+    volumes[n] = volint;
   gst_mixer_set_volume (mixer, track, volumes);
+
   g_free (volumes);
 }
 
@@ -1025,6 +1060,7 @@ cb_volume (GtkAdjustment *adj,
 
   applet->lock = FALSE;
   applet->force_next_update = TRUE;
+  gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (data), FALSE, v, -1);
 }
 
 /*
@@ -1035,31 +1071,39 @@ cb_volume (GtkAdjustment *adj,
 
 static gboolean
 gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
-			     gboolean           force_refresh)
+			     gboolean           force_refresh,
+			     gdouble            volume,
+			     gint               mute)
 {
   BonoboUIComponent *component;
   GdkPixbuf *pixbuf;
   gint n;
-  gdouble volume;
-  gboolean mute, did_change;
+  gboolean show_mute, did_change;
   gchar *tooltip_str;
   GstMixerTrack *first_track;
   GString *track_names;
   GList *iter;
 
+  show_mute = 0;
+
   if (!applet->mixer) {
     n = 0;
-    mute = FALSE;
+    show_mute = 0;
+    mute = 0;
   } else if (!applet->tracks) {
     return FALSE;
   } else {
-    /* only first track */
     first_track = g_list_first (applet->tracks)->data;
-    volume = gnome_volume_applet_get_volume (applet->mixer, first_track);
-    mute = GST_MIXER_TRACK_HAS_FLAG (first_track,
-				     GST_MIXER_TRACK_MUTE);
-    if (volume <= 0) {
-	mute = TRUE;
+    if (volume == -1) {
+      /* only first track */
+      volume = gnome_volume_applet_get_volume (applet->mixer, first_track);
+    }
+    if (mute == -1) {
+      mute = GST_MIXER_TRACK_HAS_FLAG (first_track,
+				       GST_MIXER_TRACK_MUTE) ? 1 : 0;
+    }
+    if (volume <= 0 || mute) {
+	show_mute = 1;
 	n = 0;
     }
     else {
@@ -1077,7 +1121,7 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
   applet->force_next_update = FALSE;
 
   if (did_change) {
-    if (mute) {
+    if (show_mute) {
       pixbuf = applet->pix[0];
     } else {
       pixbuf = applet->pix[n];
@@ -1101,7 +1145,7 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
       track_names = g_string_append (track_names, track->label);
   }
 
-  if (mute) {
+  if (show_mute) {
     tooltip_str = g_strdup_printf (_("%s: muted"), track_names->str);
   } else {
     /* Translator comment: I'm not all too sure if this makes sense
@@ -1117,8 +1161,9 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
   g_free (tooltip_str);
 
   applet->lock = TRUE;
-  if (volume != 0)
-    gtk_range_set_value (applet->dock->scale, volume);
+  if (volume != 0) {
+    gtk_adjustment_set_value (applet->adjustment, volume);
+  }
   applet->lock = FALSE;
 
   /* update mute status (bonobo) */
@@ -1129,6 +1174,52 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
   return did_change;
 }
 
+static void
+cb_notify_message (GstBus *bus, GstMessage *message, gpointer data)
+{
+  GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (data);
+  GstMixerMessageType type;
+  GstMixerTrack *first_track;
+  GstMixerTrack *track = NULL;
+  gint mute;
+  gdouble volume;
+
+  if (applet->tracks == NULL || 
+      GST_MESSAGE_SRC (message) != GST_OBJECT (applet->mixer)) {
+    /* No tracks, or not from our mixer - can't update anything anyway */
+    return; 
+  }
+
+  volume = mute = -1;
+
+  first_track = g_list_first (applet->tracks)->data;
+
+  /* This code only calls refresh if the first_track changes, because the
+   * refresh code only retrieves the current value from that track anyway */
+  type = gst_mixer_message_get_type (message);
+  if (type == GST_MIXER_MESSAGE_MUTE_TOGGLED) {
+    gboolean muted;
+    gst_mixer_message_parse_mute_toggled (message, &track, &muted);
+    mute = muted ? 1 : 0;
+  }
+  else if (type == GST_MIXER_MESSAGE_VOLUME_CHANGED) {
+    gint n, num_channels, *vols;
+    volume = 0.0;
+
+    gst_mixer_message_parse_volume_changed (message, &track, &vols, &num_channels);
+    for (n = 0; n < num_channels; n++)
+      volume += vols[n];
+    volume /= track->num_channels;
+    volume = 100 * volume / (track->max_volume - track->min_volume);
+  } else
+  {
+    return;
+  }
+
+  if (first_track == track)
+    gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (data), FALSE, volume, mute);
+}
+
 static gboolean
 cb_check (gpointer data)
 {
@@ -1146,7 +1237,7 @@ cb_check (gpointer data)
    */
   if (time_counter % timeout == 0 || recent_change) {
      did_change = gnome_volume_applet_refresh (GNOME_VOLUME_APPLET (data),
-                                               FALSE);
+                                               FALSE, -1, -1);
 
      /*
       * If a change was done, set recent_change so that the update is
@@ -1216,17 +1307,14 @@ cb_gconf (GConfClient *client,
 
           if (new_element != old_element) {
             /* change element */
-#ifdef HAVE_GST10
             gst_element_set_state (item->data, GST_STATE_READY);
             if (gst_element_get_state (item->data, NULL, NULL, -1) != GST_STATE_CHANGE_SUCCESS)
-#else
-            if (gst_element_set_state (item->data, GST_STATE_READY) != GST_STATE_SUCCESS)
-#endif
               continue;
 	    
             /* save */
             gst_object_replace ((GstObject **) &applet->mixer, item->data);
             gst_element_set_state (old_element, GST_STATE_NULL);
+	    gnome_volume_applet_setup_timeout (applet);
             newdevice = TRUE;
           }
           break;
@@ -1240,7 +1328,6 @@ cb_gconf (GConfClient *client,
       }
 
       if (active_tracks) {
-        GtkObject *adj;
 	GstMixerTrack *first_track;
 
 	/* copy the newly created track list over to the main list */
@@ -1250,16 +1337,10 @@ cb_gconf (GConfClient *client,
 	first_track = g_list_first (active_tracks)->data;
 
         /* dock */
-	adj = gtk_adjustment_new (50, 0, 100, 4, 10, 0);
-	gtk_adjustment_set_value (GTK_ADJUSTMENT (adj), 
+	gtk_adjustment_set_value (applet->adjustment, 
 				  gnome_volume_applet_get_volume (applet->mixer, 
 								  first_track));
 
-        gnome_volume_applet_dock_change (applet->dock,
-					 GTK_ADJUSTMENT (adj));
-        g_signal_connect (adj, "value-changed",
-			  G_CALLBACK (cb_volume), applet);
-	
         /* if preferences window is open, update */
 	if (applet->prefs) {
           gnome_volume_applet_preferences_change (
@@ -1296,9 +1377,10 @@ cb_verb (BonoboUIComponent *uic,
   } else if (!strcmp (verbname, "Help")) {
     GError *error = NULL;
 
-    gnome_help_display_on_screen ("mixer_applet2", NULL,
-				  gtk_widget_get_screen (GTK_WIDGET (applet)),
-				  &error);
+    gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (applet)),
+		  "ghelp:mixer_applet2",
+		  gtk_get_current_event_time (),
+		  &error);
 
     if (error) {
       GtkWidget *dialog;
@@ -1319,11 +1401,7 @@ cb_verb (BonoboUIComponent *uic,
 
     char *comments = g_strdup_printf ("%s\n\n%s",
 		    _("Volume control for your GNOME Panel."),
-#ifdef HAVE_GST10
 		    _("Using GStreamer 0.10.")
-#else
-		    _("Using GStreamer 0.8.")
-#endif
 		    );
 
     gtk_show_about_dialog (NULL,
@@ -1387,7 +1465,7 @@ cb_theme_change (GtkIconTheme *icon_theme,
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (data);
 
   init_pixbufs (applet);
-  gnome_volume_applet_refresh (applet, TRUE);
+  gnome_volume_applet_refresh (applet, TRUE, -1, -1);
 }
 
 /*
